/* Generated automatically. */
static const char configuration_arguments[] = "/var/lib/managarm-buildenv/src/ports/gcc/configure --with-pkgversion=managarm --with-bugurl=https://github.com/managarm/bootstrap-managarm/issues/ --prefix=/var/lib/managarm-buildenv/build/tools/system-gcc --target=riscv64-managarm --with-sysroot=/var/lib/managarm-buildenv/build/system-root --enable-languages=c,c++,lto --enable-initfini-array --enable-libstdcxx-filesystem-ts --enable-linker-build-id --enable-default-pie --disable-fixincludes CFLAGS=-O2 CXXFLAGS=-O2";
static const char thread_model[] = "posix";

static const struct {
  const char *name, *value;
} configure_default_options[] = { { "abi", "lp64d" }, { "arch", "rv64imafdc_zicsr_zifencei" }, { "isa_spec", "20191213" } };
