/* Generated automatically from machmode.def and config/riscv/riscv-modes.def
   by genmodes.  */

#ifndef GCC_INSN_MODES_INLINE_H
#define GCC_INSN_MODES_INLINE_H

#if !defined (USED_FOR_TARGET) && GCC_VERSION >= 4001

#ifdef __cplusplus
inline __attribute__((__always_inline__))
#else
extern __inline__ __attribute__((__always_inline__, __gnu_inline__))
#endif
poly_uint16
mode_size_inline (machine_mode mode)
{
  extern poly_uint16 mode_size[NUM_MACHINE_MODES];
  gcc_assert (mode >= 0 && mode < NUM_MACHINE_MODES);
  switch (mode)
    {
    case E_VOIDmode: return 0;
    case E_BLKmode: return 0;
    case E_CCmode: return 4;
    case E_BImode: return 1;
    case E_QImode: return 1;
    case E_HImode: return 2;
    case E_SImode: return 4;
    case E_DImode: return 8;
    case E_TImode: return 16;
    case E_QQmode: return 1;
    case E_HQmode: return 2;
    case E_SQmode: return 4;
    case E_DQmode: return 8;
    case E_TQmode: return 16;
    case E_UQQmode: return 1;
    case E_UHQmode: return 2;
    case E_USQmode: return 4;
    case E_UDQmode: return 8;
    case E_UTQmode: return 16;
    case E_HAmode: return 2;
    case E_SAmode: return 4;
    case E_DAmode: return 8;
    case E_TAmode: return 16;
    case E_UHAmode: return 2;
    case E_USAmode: return 4;
    case E_UDAmode: return 8;
    case E_UTAmode: return 16;
    case E_HFmode: return 2;
    case E_SFmode: return 4;
    case E_DFmode: return 8;
    case E_TFmode: return 16;
    case E_SDmode: return 4;
    case E_DDmode: return 8;
    case E_TDmode: return 16;
    case E_CQImode: return 2;
    case E_CHImode: return 4;
    case E_CSImode: return 8;
    case E_CDImode: return 16;
    case E_CTImode: return 32;
    case E_HCmode: return 4;
    case E_SCmode: return 8;
    case E_DCmode: return 16;
    case E_TCmode: return 32;
    case E_V1BImode: return 1;
    case E_V2BImode: return 1;
    case E_V4BImode: return 1;
    case E_V8BImode: return 1;
    case E_V16BImode: return 2;
    case E_V32BImode: return 4;
    case E_V64BImode: return 8;
    case E_V128BImode: return 16;
    case E_V256BImode: return 32;
    case E_V512BImode: return 64;
    case E_V1024BImode: return 128;
    case E_V2048BImode: return 256;
    case E_V4096BImode: return 512;
    case E_V1QImode: return 1;
    case E_V2QImode: return 2;
    case E_V1HImode: return 2;
    case E_V4QImode: return 4;
    case E_V2HImode: return 4;
    case E_V1SImode: return 4;
    case E_V8QImode: return 8;
    case E_V4HImode: return 8;
    case E_V2SImode: return 8;
    case E_V1DImode: return 8;
    case E_V16QImode: return 16;
    case E_V8HImode: return 16;
    case E_V4SImode: return 16;
    case E_V2DImode: return 16;
    case E_V32QImode: return 32;
    case E_V16HImode: return 32;
    case E_V8SImode: return 32;
    case E_V4DImode: return 32;
    case E_V64QImode: return 64;
    case E_V32HImode: return 64;
    case E_V16SImode: return 64;
    case E_V8DImode: return 64;
    case E_V128QImode: return 128;
    case E_V64HImode: return 128;
    case E_V32SImode: return 128;
    case E_V16DImode: return 128;
    case E_V256QImode: return 256;
    case E_V128HImode: return 256;
    case E_V64SImode: return 256;
    case E_V32DImode: return 256;
    case E_V512QImode: return 512;
    case E_V256HImode: return 512;
    case E_V128SImode: return 512;
    case E_V64DImode: return 512;
    case E_V1024QImode: return 1024;
    case E_V512HImode: return 1024;
    case E_V256SImode: return 1024;
    case E_V128DImode: return 1024;
    case E_V2048QImode: return 2048;
    case E_V1024HImode: return 2048;
    case E_V512SImode: return 2048;
    case E_V256DImode: return 2048;
    case E_V4096QImode: return 4096;
    case E_V2048HImode: return 4096;
    case E_V1024SImode: return 4096;
    case E_V512DImode: return 4096;
    case E_V1HFmode: return 2;
    case E_V2HFmode: return 4;
    case E_V1SFmode: return 4;
    case E_V4HFmode: return 8;
    case E_V2SFmode: return 8;
    case E_V1DFmode: return 8;
    case E_V8HFmode: return 16;
    case E_V4SFmode: return 16;
    case E_V2DFmode: return 16;
    case E_V16HFmode: return 32;
    case E_V8SFmode: return 32;
    case E_V4DFmode: return 32;
    case E_V32HFmode: return 64;
    case E_V16SFmode: return 64;
    case E_V8DFmode: return 64;
    case E_V64HFmode: return 128;
    case E_V32SFmode: return 128;
    case E_V16DFmode: return 128;
    case E_V128HFmode: return 256;
    case E_V64SFmode: return 256;
    case E_V32DFmode: return 256;
    case E_V256HFmode: return 512;
    case E_V128SFmode: return 512;
    case E_V64DFmode: return 512;
    case E_V512HFmode: return 1024;
    case E_V256SFmode: return 1024;
    case E_V128DFmode: return 1024;
    case E_V1024HFmode: return 2048;
    case E_V512SFmode: return 2048;
    case E_V256DFmode: return 2048;
    case E_V2048HFmode: return 4096;
    case E_V1024SFmode: return 4096;
    case E_V512DFmode: return 4096;
    default: return mode_size[mode];
    }
}

#ifdef __cplusplus
inline __attribute__((__always_inline__))
#else
extern __inline__ __attribute__((__always_inline__, __gnu_inline__))
#endif
poly_uint16
mode_nunits_inline (machine_mode mode)
{
  extern poly_uint16 mode_nunits[NUM_MACHINE_MODES];
  switch (mode)
    {
    case E_VOIDmode: return 0;
    case E_BLKmode: return 0;
    case E_CCmode: return 1;
    case E_BImode: return 1;
    case E_QImode: return 1;
    case E_HImode: return 1;
    case E_SImode: return 1;
    case E_DImode: return 1;
    case E_TImode: return 1;
    case E_QQmode: return 1;
    case E_HQmode: return 1;
    case E_SQmode: return 1;
    case E_DQmode: return 1;
    case E_TQmode: return 1;
    case E_UQQmode: return 1;
    case E_UHQmode: return 1;
    case E_USQmode: return 1;
    case E_UDQmode: return 1;
    case E_UTQmode: return 1;
    case E_HAmode: return 1;
    case E_SAmode: return 1;
    case E_DAmode: return 1;
    case E_TAmode: return 1;
    case E_UHAmode: return 1;
    case E_USAmode: return 1;
    case E_UDAmode: return 1;
    case E_UTAmode: return 1;
    case E_HFmode: return 1;
    case E_SFmode: return 1;
    case E_DFmode: return 1;
    case E_TFmode: return 1;
    case E_SDmode: return 1;
    case E_DDmode: return 1;
    case E_TDmode: return 1;
    case E_CQImode: return 2;
    case E_CHImode: return 2;
    case E_CSImode: return 2;
    case E_CDImode: return 2;
    case E_CTImode: return 2;
    case E_HCmode: return 2;
    case E_SCmode: return 2;
    case E_DCmode: return 2;
    case E_TCmode: return 2;
    case E_V1BImode: return 1;
    case E_V2BImode: return 2;
    case E_V4BImode: return 4;
    case E_V8BImode: return 8;
    case E_V16BImode: return 16;
    case E_V32BImode: return 32;
    case E_V64BImode: return 64;
    case E_V128BImode: return 128;
    case E_V256BImode: return 256;
    case E_V512BImode: return 512;
    case E_V1024BImode: return 1024;
    case E_V2048BImode: return 2048;
    case E_V4096BImode: return 4096;
    case E_V1QImode: return 1;
    case E_V2QImode: return 2;
    case E_V1HImode: return 1;
    case E_V4QImode: return 4;
    case E_V2HImode: return 2;
    case E_V1SImode: return 1;
    case E_V8QImode: return 8;
    case E_V4HImode: return 4;
    case E_V2SImode: return 2;
    case E_V1DImode: return 1;
    case E_V16QImode: return 16;
    case E_V8HImode: return 8;
    case E_V4SImode: return 4;
    case E_V2DImode: return 2;
    case E_V32QImode: return 32;
    case E_V16HImode: return 16;
    case E_V8SImode: return 8;
    case E_V4DImode: return 4;
    case E_V64QImode: return 64;
    case E_V32HImode: return 32;
    case E_V16SImode: return 16;
    case E_V8DImode: return 8;
    case E_V128QImode: return 128;
    case E_V64HImode: return 64;
    case E_V32SImode: return 32;
    case E_V16DImode: return 16;
    case E_V256QImode: return 256;
    case E_V128HImode: return 128;
    case E_V64SImode: return 64;
    case E_V32DImode: return 32;
    case E_V512QImode: return 512;
    case E_V256HImode: return 256;
    case E_V128SImode: return 128;
    case E_V64DImode: return 64;
    case E_V1024QImode: return 1024;
    case E_V512HImode: return 512;
    case E_V256SImode: return 256;
    case E_V128DImode: return 128;
    case E_V2048QImode: return 2048;
    case E_V1024HImode: return 1024;
    case E_V512SImode: return 512;
    case E_V256DImode: return 256;
    case E_V4096QImode: return 4096;
    case E_V2048HImode: return 2048;
    case E_V1024SImode: return 1024;
    case E_V512DImode: return 512;
    case E_V1HFmode: return 1;
    case E_V2HFmode: return 2;
    case E_V1SFmode: return 1;
    case E_V4HFmode: return 4;
    case E_V2SFmode: return 2;
    case E_V1DFmode: return 1;
    case E_V8HFmode: return 8;
    case E_V4SFmode: return 4;
    case E_V2DFmode: return 2;
    case E_V16HFmode: return 16;
    case E_V8SFmode: return 8;
    case E_V4DFmode: return 4;
    case E_V32HFmode: return 32;
    case E_V16SFmode: return 16;
    case E_V8DFmode: return 8;
    case E_V64HFmode: return 64;
    case E_V32SFmode: return 32;
    case E_V16DFmode: return 16;
    case E_V128HFmode: return 128;
    case E_V64SFmode: return 64;
    case E_V32DFmode: return 32;
    case E_V256HFmode: return 256;
    case E_V128SFmode: return 128;
    case E_V64DFmode: return 64;
    case E_V512HFmode: return 512;
    case E_V256SFmode: return 256;
    case E_V128DFmode: return 128;
    case E_V1024HFmode: return 1024;
    case E_V512SFmode: return 512;
    case E_V256DFmode: return 256;
    case E_V2048HFmode: return 2048;
    case E_V1024SFmode: return 1024;
    case E_V512DFmode: return 512;
    default: return mode_nunits[mode];
    }
}

#ifdef __cplusplus
inline __attribute__((__always_inline__))
#else
extern __inline__ __attribute__((__always_inline__, __gnu_inline__))
#endif
unsigned short
mode_inner_inline (machine_mode mode)
{
  extern const unsigned short mode_inner[NUM_MACHINE_MODES];
  gcc_assert (mode >= 0 && mode < NUM_MACHINE_MODES);
  switch (mode)
    {
    case E_VOIDmode: return E_VOIDmode;
    case E_BLKmode: return E_BLKmode;
    case E_CCmode: return E_CCmode;
    case E_BImode: return E_BImode;
    case E_QImode: return E_QImode;
    case E_HImode: return E_HImode;
    case E_SImode: return E_SImode;
    case E_DImode: return E_DImode;
    case E_TImode: return E_TImode;
    case E_QQmode: return E_QQmode;
    case E_HQmode: return E_HQmode;
    case E_SQmode: return E_SQmode;
    case E_DQmode: return E_DQmode;
    case E_TQmode: return E_TQmode;
    case E_UQQmode: return E_UQQmode;
    case E_UHQmode: return E_UHQmode;
    case E_USQmode: return E_USQmode;
    case E_UDQmode: return E_UDQmode;
    case E_UTQmode: return E_UTQmode;
    case E_HAmode: return E_HAmode;
    case E_SAmode: return E_SAmode;
    case E_DAmode: return E_DAmode;
    case E_TAmode: return E_TAmode;
    case E_UHAmode: return E_UHAmode;
    case E_USAmode: return E_USAmode;
    case E_UDAmode: return E_UDAmode;
    case E_UTAmode: return E_UTAmode;
    case E_HFmode: return E_HFmode;
    case E_SFmode: return E_SFmode;
    case E_DFmode: return E_DFmode;
    case E_TFmode: return E_TFmode;
    case E_SDmode: return E_SDmode;
    case E_DDmode: return E_DDmode;
    case E_TDmode: return E_TDmode;
    case E_CQImode: return E_QImode;
    case E_CHImode: return E_HImode;
    case E_CSImode: return E_SImode;
    case E_CDImode: return E_DImode;
    case E_CTImode: return E_TImode;
    case E_HCmode: return E_HFmode;
    case E_SCmode: return E_SFmode;
    case E_DCmode: return E_DFmode;
    case E_TCmode: return E_TFmode;
    case E_RVVMF8BImode: return E_BImode;
    case E_RVVMF16BImode: return E_BImode;
    case E_RVVMF32BImode: return E_BImode;
    case E_RVVMF64BImode: return E_BImode;
    case E_V1BImode: return E_BImode;
    case E_V2BImode: return E_BImode;
    case E_V4BImode: return E_BImode;
    case E_V8BImode: return E_BImode;
    case E_RVVMF4BImode: return E_BImode;
    case E_V16BImode: return E_BImode;
    case E_RVVMF2BImode: return E_BImode;
    case E_V32BImode: return E_BImode;
    case E_RVVM1BImode: return E_BImode;
    case E_V64BImode: return E_BImode;
    case E_V128BImode: return E_BImode;
    case E_V256BImode: return E_BImode;
    case E_V512BImode: return E_BImode;
    case E_V1024BImode: return E_BImode;
    case E_V2048BImode: return E_BImode;
    case E_V4096BImode: return E_BImode;
    case E_RVVM1QImode: return E_QImode;
    case E_RVVM2QImode: return E_QImode;
    case E_RVVMF2QImode: return E_QImode;
    case E_RVVM1HImode: return E_HImode;
    case E_RVVM4QImode: return E_QImode;
    case E_RVVMF4QImode: return E_QImode;
    case E_RVVM2HImode: return E_HImode;
    case E_RVVMF2HImode: return E_HImode;
    case E_RVVM1SImode: return E_SImode;
    case E_RVVM8QImode: return E_QImode;
    case E_RVVMF8QImode: return E_QImode;
    case E_RVVM4HImode: return E_HImode;
    case E_RVVMF4HImode: return E_HImode;
    case E_RVVM2SImode: return E_SImode;
    case E_RVVMF2SImode: return E_SImode;
    case E_RVVM1DImode: return E_DImode;
    case E_RVVM8HImode: return E_HImode;
    case E_RVVM4SImode: return E_SImode;
    case E_RVVM2DImode: return E_DImode;
    case E_RVVM8SImode: return E_SImode;
    case E_RVVM4DImode: return E_DImode;
    case E_RVVM8DImode: return E_DImode;
    case E_V1QImode: return E_QImode;
    case E_RVVMF8x2QImode: return E_QImode;
    case E_RVVMF4x2QImode: return E_QImode;
    case E_RVVMF2x2QImode: return E_QImode;
    case E_RVVM1x2QImode: return E_QImode;
    case E_RVVM2x2QImode: return E_QImode;
    case E_RVVM4x2QImode: return E_QImode;
    case E_V2QImode: return E_QImode;
    case E_V1HImode: return E_HImode;
    case E_RVVMF8x3QImode: return E_QImode;
    case E_RVVMF4x3QImode: return E_QImode;
    case E_RVVMF2x3QImode: return E_QImode;
    case E_RVVM1x3QImode: return E_QImode;
    case E_RVVM2x3QImode: return E_QImode;
    case E_RVVMF8x4QImode: return E_QImode;
    case E_RVVMF4x4QImode: return E_QImode;
    case E_RVVMF2x4QImode: return E_QImode;
    case E_RVVM1x4QImode: return E_QImode;
    case E_RVVM2x4QImode: return E_QImode;
    case E_V4QImode: return E_QImode;
    case E_RVVMF4x2HImode: return E_HImode;
    case E_RVVMF2x2HImode: return E_HImode;
    case E_RVVM1x2HImode: return E_HImode;
    case E_RVVM2x2HImode: return E_HImode;
    case E_RVVM4x2HImode: return E_HImode;
    case E_V2HImode: return E_HImode;
    case E_V1SImode: return E_SImode;
    case E_RVVMF8x5QImode: return E_QImode;
    case E_RVVMF4x5QImode: return E_QImode;
    case E_RVVMF2x5QImode: return E_QImode;
    case E_RVVM1x5QImode: return E_QImode;
    case E_RVVMF8x6QImode: return E_QImode;
    case E_RVVMF4x6QImode: return E_QImode;
    case E_RVVMF2x6QImode: return E_QImode;
    case E_RVVM1x6QImode: return E_QImode;
    case E_RVVMF4x3HImode: return E_HImode;
    case E_RVVMF2x3HImode: return E_HImode;
    case E_RVVM1x3HImode: return E_HImode;
    case E_RVVM2x3HImode: return E_HImode;
    case E_RVVMF8x7QImode: return E_QImode;
    case E_RVVMF4x7QImode: return E_QImode;
    case E_RVVMF2x7QImode: return E_QImode;
    case E_RVVM1x7QImode: return E_QImode;
    case E_RVVMF8x8QImode: return E_QImode;
    case E_RVVMF4x8QImode: return E_QImode;
    case E_RVVMF2x8QImode: return E_QImode;
    case E_RVVM1x8QImode: return E_QImode;
    case E_V8QImode: return E_QImode;
    case E_RVVMF4x4HImode: return E_HImode;
    case E_RVVMF2x4HImode: return E_HImode;
    case E_RVVM1x4HImode: return E_HImode;
    case E_RVVM2x4HImode: return E_HImode;
    case E_V4HImode: return E_HImode;
    case E_RVVMF2x2SImode: return E_SImode;
    case E_RVVM1x2SImode: return E_SImode;
    case E_RVVM2x2SImode: return E_SImode;
    case E_RVVM4x2SImode: return E_SImode;
    case E_V2SImode: return E_SImode;
    case E_V1DImode: return E_DImode;
    case E_RVVMF4x5HImode: return E_HImode;
    case E_RVVMF2x5HImode: return E_HImode;
    case E_RVVM1x5HImode: return E_HImode;
    case E_RVVMF4x6HImode: return E_HImode;
    case E_RVVMF2x6HImode: return E_HImode;
    case E_RVVM1x6HImode: return E_HImode;
    case E_RVVMF2x3SImode: return E_SImode;
    case E_RVVM1x3SImode: return E_SImode;
    case E_RVVM2x3SImode: return E_SImode;
    case E_RVVMF4x7HImode: return E_HImode;
    case E_RVVMF2x7HImode: return E_HImode;
    case E_RVVM1x7HImode: return E_HImode;
    case E_V16QImode: return E_QImode;
    case E_RVVMF4x8HImode: return E_HImode;
    case E_RVVMF2x8HImode: return E_HImode;
    case E_RVVM1x8HImode: return E_HImode;
    case E_V8HImode: return E_HImode;
    case E_RVVMF2x4SImode: return E_SImode;
    case E_RVVM1x4SImode: return E_SImode;
    case E_RVVM2x4SImode: return E_SImode;
    case E_V4SImode: return E_SImode;
    case E_RVVM1x2DImode: return E_DImode;
    case E_RVVM2x2DImode: return E_DImode;
    case E_RVVM4x2DImode: return E_DImode;
    case E_V2DImode: return E_DImode;
    case E_RVVMF2x5SImode: return E_SImode;
    case E_RVVM1x5SImode: return E_SImode;
    case E_RVVMF2x6SImode: return E_SImode;
    case E_RVVM1x6SImode: return E_SImode;
    case E_RVVM1x3DImode: return E_DImode;
    case E_RVVM2x3DImode: return E_DImode;
    case E_RVVMF2x7SImode: return E_SImode;
    case E_RVVM1x7SImode: return E_SImode;
    case E_V32QImode: return E_QImode;
    case E_V16HImode: return E_HImode;
    case E_RVVMF2x8SImode: return E_SImode;
    case E_RVVM1x8SImode: return E_SImode;
    case E_V8SImode: return E_SImode;
    case E_RVVM1x4DImode: return E_DImode;
    case E_RVVM2x4DImode: return E_DImode;
    case E_V4DImode: return E_DImode;
    case E_RVVM1x5DImode: return E_DImode;
    case E_RVVM1x6DImode: return E_DImode;
    case E_RVVM1x7DImode: return E_DImode;
    case E_V64QImode: return E_QImode;
    case E_V32HImode: return E_HImode;
    case E_V16SImode: return E_SImode;
    case E_RVVM1x8DImode: return E_DImode;
    case E_V8DImode: return E_DImode;
    case E_V128QImode: return E_QImode;
    case E_V64HImode: return E_HImode;
    case E_V32SImode: return E_SImode;
    case E_V16DImode: return E_DImode;
    case E_V256QImode: return E_QImode;
    case E_V128HImode: return E_HImode;
    case E_V64SImode: return E_SImode;
    case E_V32DImode: return E_DImode;
    case E_V512QImode: return E_QImode;
    case E_V256HImode: return E_HImode;
    case E_V128SImode: return E_SImode;
    case E_V64DImode: return E_DImode;
    case E_V1024QImode: return E_QImode;
    case E_V512HImode: return E_HImode;
    case E_V256SImode: return E_SImode;
    case E_V128DImode: return E_DImode;
    case E_V2048QImode: return E_QImode;
    case E_V1024HImode: return E_HImode;
    case E_V512SImode: return E_SImode;
    case E_V256DImode: return E_DImode;
    case E_V4096QImode: return E_QImode;
    case E_V2048HImode: return E_HImode;
    case E_V1024SImode: return E_SImode;
    case E_V512DImode: return E_DImode;
    case E_RVVM1HFmode: return E_HFmode;
    case E_RVVM2HFmode: return E_HFmode;
    case E_RVVMF2HFmode: return E_HFmode;
    case E_RVVM1SFmode: return E_SFmode;
    case E_RVVM4HFmode: return E_HFmode;
    case E_RVVMF4HFmode: return E_HFmode;
    case E_RVVM2SFmode: return E_SFmode;
    case E_RVVMF2SFmode: return E_SFmode;
    case E_RVVM1DFmode: return E_DFmode;
    case E_RVVM8HFmode: return E_HFmode;
    case E_RVVM4SFmode: return E_SFmode;
    case E_RVVM2DFmode: return E_DFmode;
    case E_RVVM8SFmode: return E_SFmode;
    case E_RVVM4DFmode: return E_DFmode;
    case E_RVVM8DFmode: return E_DFmode;
    case E_V1HFmode: return E_HFmode;
    case E_RVVMF4x2HFmode: return E_HFmode;
    case E_RVVMF2x2HFmode: return E_HFmode;
    case E_RVVM1x2HFmode: return E_HFmode;
    case E_RVVM2x2HFmode: return E_HFmode;
    case E_RVVM4x2HFmode: return E_HFmode;
    case E_V2HFmode: return E_HFmode;
    case E_V1SFmode: return E_SFmode;
    case E_RVVMF4x3HFmode: return E_HFmode;
    case E_RVVMF2x3HFmode: return E_HFmode;
    case E_RVVM1x3HFmode: return E_HFmode;
    case E_RVVM2x3HFmode: return E_HFmode;
    case E_RVVMF4x4HFmode: return E_HFmode;
    case E_RVVMF2x4HFmode: return E_HFmode;
    case E_RVVM1x4HFmode: return E_HFmode;
    case E_RVVM2x4HFmode: return E_HFmode;
    case E_V4HFmode: return E_HFmode;
    case E_RVVMF2x2SFmode: return E_SFmode;
    case E_RVVM1x2SFmode: return E_SFmode;
    case E_RVVM2x2SFmode: return E_SFmode;
    case E_RVVM4x2SFmode: return E_SFmode;
    case E_V2SFmode: return E_SFmode;
    case E_V1DFmode: return E_DFmode;
    case E_RVVMF4x5HFmode: return E_HFmode;
    case E_RVVMF2x5HFmode: return E_HFmode;
    case E_RVVM1x5HFmode: return E_HFmode;
    case E_RVVMF4x6HFmode: return E_HFmode;
    case E_RVVMF2x6HFmode: return E_HFmode;
    case E_RVVM1x6HFmode: return E_HFmode;
    case E_RVVMF2x3SFmode: return E_SFmode;
    case E_RVVM1x3SFmode: return E_SFmode;
    case E_RVVM2x3SFmode: return E_SFmode;
    case E_RVVMF4x7HFmode: return E_HFmode;
    case E_RVVMF2x7HFmode: return E_HFmode;
    case E_RVVM1x7HFmode: return E_HFmode;
    case E_RVVMF4x8HFmode: return E_HFmode;
    case E_RVVMF2x8HFmode: return E_HFmode;
    case E_RVVM1x8HFmode: return E_HFmode;
    case E_V8HFmode: return E_HFmode;
    case E_RVVMF2x4SFmode: return E_SFmode;
    case E_RVVM1x4SFmode: return E_SFmode;
    case E_RVVM2x4SFmode: return E_SFmode;
    case E_V4SFmode: return E_SFmode;
    case E_RVVM1x2DFmode: return E_DFmode;
    case E_RVVM2x2DFmode: return E_DFmode;
    case E_RVVM4x2DFmode: return E_DFmode;
    case E_V2DFmode: return E_DFmode;
    case E_RVVMF2x5SFmode: return E_SFmode;
    case E_RVVM1x5SFmode: return E_SFmode;
    case E_RVVMF2x6SFmode: return E_SFmode;
    case E_RVVM1x6SFmode: return E_SFmode;
    case E_RVVM1x3DFmode: return E_DFmode;
    case E_RVVM2x3DFmode: return E_DFmode;
    case E_RVVMF2x7SFmode: return E_SFmode;
    case E_RVVM1x7SFmode: return E_SFmode;
    case E_V16HFmode: return E_HFmode;
    case E_RVVMF2x8SFmode: return E_SFmode;
    case E_RVVM1x8SFmode: return E_SFmode;
    case E_V8SFmode: return E_SFmode;
    case E_RVVM1x4DFmode: return E_DFmode;
    case E_RVVM2x4DFmode: return E_DFmode;
    case E_V4DFmode: return E_DFmode;
    case E_RVVM1x5DFmode: return E_DFmode;
    case E_RVVM1x6DFmode: return E_DFmode;
    case E_RVVM1x7DFmode: return E_DFmode;
    case E_V32HFmode: return E_HFmode;
    case E_V16SFmode: return E_SFmode;
    case E_RVVM1x8DFmode: return E_DFmode;
    case E_V8DFmode: return E_DFmode;
    case E_V64HFmode: return E_HFmode;
    case E_V32SFmode: return E_SFmode;
    case E_V16DFmode: return E_DFmode;
    case E_V128HFmode: return E_HFmode;
    case E_V64SFmode: return E_SFmode;
    case E_V32DFmode: return E_DFmode;
    case E_V256HFmode: return E_HFmode;
    case E_V128SFmode: return E_SFmode;
    case E_V64DFmode: return E_DFmode;
    case E_V512HFmode: return E_HFmode;
    case E_V256SFmode: return E_SFmode;
    case E_V128DFmode: return E_DFmode;
    case E_V1024HFmode: return E_HFmode;
    case E_V512SFmode: return E_SFmode;
    case E_V256DFmode: return E_DFmode;
    case E_V2048HFmode: return E_HFmode;
    case E_V1024SFmode: return E_SFmode;
    case E_V512DFmode: return E_DFmode;
    default: return mode_inner[mode];
    }
}

#ifdef __cplusplus
inline __attribute__((__always_inline__))
#else
extern __inline__ __attribute__((__always_inline__, __gnu_inline__))
#endif
unsigned char
mode_unit_size_inline (machine_mode mode)
{
  extern CONST_MODE_UNIT_SIZE unsigned char mode_unit_size[NUM_MACHINE_MODES];
  gcc_assert (mode >= 0 && mode < NUM_MACHINE_MODES);
  switch (mode)
    {
    case E_VOIDmode: return 0;
    case E_BLKmode: return 0;
    case E_CCmode: return 4;
    case E_BImode: return 1;
    case E_QImode: return 1;
    case E_HImode: return 2;
    case E_SImode: return 4;
    case E_DImode: return 8;
    case E_TImode: return 16;
    case E_QQmode: return 1;
    case E_HQmode: return 2;
    case E_SQmode: return 4;
    case E_DQmode: return 8;
    case E_TQmode: return 16;
    case E_UQQmode: return 1;
    case E_UHQmode: return 2;
    case E_USQmode: return 4;
    case E_UDQmode: return 8;
    case E_UTQmode: return 16;
    case E_HAmode: return 2;
    case E_SAmode: return 4;
    case E_DAmode: return 8;
    case E_TAmode: return 16;
    case E_UHAmode: return 2;
    case E_USAmode: return 4;
    case E_UDAmode: return 8;
    case E_UTAmode: return 16;
    case E_HFmode: return 2;
    case E_SFmode: return 4;
    case E_DFmode: return 8;
    case E_TFmode: return 16;
    case E_SDmode: return 4;
    case E_DDmode: return 8;
    case E_TDmode: return 16;
    case E_CQImode: return 1;
    case E_CHImode: return 2;
    case E_CSImode: return 4;
    case E_CDImode: return 8;
    case E_CTImode: return 16;
    case E_HCmode: return 2;
    case E_SCmode: return 4;
    case E_DCmode: return 8;
    case E_TCmode: return 16;
    case E_RVVMF8BImode: return 1;
    case E_RVVMF16BImode: return 1;
    case E_RVVMF32BImode: return 1;
    case E_RVVMF64BImode: return 1;
    case E_V1BImode: return 1;
    case E_V2BImode: return 1;
    case E_V4BImode: return 1;
    case E_V8BImode: return 1;
    case E_RVVMF4BImode: return 1;
    case E_V16BImode: return 1;
    case E_RVVMF2BImode: return 1;
    case E_V32BImode: return 1;
    case E_RVVM1BImode: return 1;
    case E_V64BImode: return 1;
    case E_V128BImode: return 1;
    case E_V256BImode: return 1;
    case E_V512BImode: return 1;
    case E_V1024BImode: return 1;
    case E_V2048BImode: return 1;
    case E_V4096BImode: return 1;
    case E_RVVM1QImode: return 1;
    case E_RVVM2QImode: return 1;
    case E_RVVMF2QImode: return 1;
    case E_RVVM1HImode: return 2;
    case E_RVVM4QImode: return 1;
    case E_RVVMF4QImode: return 1;
    case E_RVVM2HImode: return 2;
    case E_RVVMF2HImode: return 2;
    case E_RVVM1SImode: return 4;
    case E_RVVM8QImode: return 1;
    case E_RVVMF8QImode: return 1;
    case E_RVVM4HImode: return 2;
    case E_RVVMF4HImode: return 2;
    case E_RVVM2SImode: return 4;
    case E_RVVMF2SImode: return 4;
    case E_RVVM1DImode: return 8;
    case E_RVVM8HImode: return 2;
    case E_RVVM4SImode: return 4;
    case E_RVVM2DImode: return 8;
    case E_RVVM8SImode: return 4;
    case E_RVVM4DImode: return 8;
    case E_RVVM8DImode: return 8;
    case E_V1QImode: return 1;
    case E_RVVMF8x2QImode: return 1;
    case E_RVVMF4x2QImode: return 1;
    case E_RVVMF2x2QImode: return 1;
    case E_RVVM1x2QImode: return 1;
    case E_RVVM2x2QImode: return 1;
    case E_RVVM4x2QImode: return 1;
    case E_V2QImode: return 1;
    case E_V1HImode: return 2;
    case E_RVVMF8x3QImode: return 1;
    case E_RVVMF4x3QImode: return 1;
    case E_RVVMF2x3QImode: return 1;
    case E_RVVM1x3QImode: return 1;
    case E_RVVM2x3QImode: return 1;
    case E_RVVMF8x4QImode: return 1;
    case E_RVVMF4x4QImode: return 1;
    case E_RVVMF2x4QImode: return 1;
    case E_RVVM1x4QImode: return 1;
    case E_RVVM2x4QImode: return 1;
    case E_V4QImode: return 1;
    case E_RVVMF4x2HImode: return 2;
    case E_RVVMF2x2HImode: return 2;
    case E_RVVM1x2HImode: return 2;
    case E_RVVM2x2HImode: return 2;
    case E_RVVM4x2HImode: return 2;
    case E_V2HImode: return 2;
    case E_V1SImode: return 4;
    case E_RVVMF8x5QImode: return 1;
    case E_RVVMF4x5QImode: return 1;
    case E_RVVMF2x5QImode: return 1;
    case E_RVVM1x5QImode: return 1;
    case E_RVVMF8x6QImode: return 1;
    case E_RVVMF4x6QImode: return 1;
    case E_RVVMF2x6QImode: return 1;
    case E_RVVM1x6QImode: return 1;
    case E_RVVMF4x3HImode: return 2;
    case E_RVVMF2x3HImode: return 2;
    case E_RVVM1x3HImode: return 2;
    case E_RVVM2x3HImode: return 2;
    case E_RVVMF8x7QImode: return 1;
    case E_RVVMF4x7QImode: return 1;
    case E_RVVMF2x7QImode: return 1;
    case E_RVVM1x7QImode: return 1;
    case E_RVVMF8x8QImode: return 1;
    case E_RVVMF4x8QImode: return 1;
    case E_RVVMF2x8QImode: return 1;
    case E_RVVM1x8QImode: return 1;
    case E_V8QImode: return 1;
    case E_RVVMF4x4HImode: return 2;
    case E_RVVMF2x4HImode: return 2;
    case E_RVVM1x4HImode: return 2;
    case E_RVVM2x4HImode: return 2;
    case E_V4HImode: return 2;
    case E_RVVMF2x2SImode: return 4;
    case E_RVVM1x2SImode: return 4;
    case E_RVVM2x2SImode: return 4;
    case E_RVVM4x2SImode: return 4;
    case E_V2SImode: return 4;
    case E_V1DImode: return 8;
    case E_RVVMF4x5HImode: return 2;
    case E_RVVMF2x5HImode: return 2;
    case E_RVVM1x5HImode: return 2;
    case E_RVVMF4x6HImode: return 2;
    case E_RVVMF2x6HImode: return 2;
    case E_RVVM1x6HImode: return 2;
    case E_RVVMF2x3SImode: return 4;
    case E_RVVM1x3SImode: return 4;
    case E_RVVM2x3SImode: return 4;
    case E_RVVMF4x7HImode: return 2;
    case E_RVVMF2x7HImode: return 2;
    case E_RVVM1x7HImode: return 2;
    case E_V16QImode: return 1;
    case E_RVVMF4x8HImode: return 2;
    case E_RVVMF2x8HImode: return 2;
    case E_RVVM1x8HImode: return 2;
    case E_V8HImode: return 2;
    case E_RVVMF2x4SImode: return 4;
    case E_RVVM1x4SImode: return 4;
    case E_RVVM2x4SImode: return 4;
    case E_V4SImode: return 4;
    case E_RVVM1x2DImode: return 8;
    case E_RVVM2x2DImode: return 8;
    case E_RVVM4x2DImode: return 8;
    case E_V2DImode: return 8;
    case E_RVVMF2x5SImode: return 4;
    case E_RVVM1x5SImode: return 4;
    case E_RVVMF2x6SImode: return 4;
    case E_RVVM1x6SImode: return 4;
    case E_RVVM1x3DImode: return 8;
    case E_RVVM2x3DImode: return 8;
    case E_RVVMF2x7SImode: return 4;
    case E_RVVM1x7SImode: return 4;
    case E_V32QImode: return 1;
    case E_V16HImode: return 2;
    case E_RVVMF2x8SImode: return 4;
    case E_RVVM1x8SImode: return 4;
    case E_V8SImode: return 4;
    case E_RVVM1x4DImode: return 8;
    case E_RVVM2x4DImode: return 8;
    case E_V4DImode: return 8;
    case E_RVVM1x5DImode: return 8;
    case E_RVVM1x6DImode: return 8;
    case E_RVVM1x7DImode: return 8;
    case E_V64QImode: return 1;
    case E_V32HImode: return 2;
    case E_V16SImode: return 4;
    case E_RVVM1x8DImode: return 8;
    case E_V8DImode: return 8;
    case E_V128QImode: return 1;
    case E_V64HImode: return 2;
    case E_V32SImode: return 4;
    case E_V16DImode: return 8;
    case E_V256QImode: return 1;
    case E_V128HImode: return 2;
    case E_V64SImode: return 4;
    case E_V32DImode: return 8;
    case E_V512QImode: return 1;
    case E_V256HImode: return 2;
    case E_V128SImode: return 4;
    case E_V64DImode: return 8;
    case E_V1024QImode: return 1;
    case E_V512HImode: return 2;
    case E_V256SImode: return 4;
    case E_V128DImode: return 8;
    case E_V2048QImode: return 1;
    case E_V1024HImode: return 2;
    case E_V512SImode: return 4;
    case E_V256DImode: return 8;
    case E_V4096QImode: return 1;
    case E_V2048HImode: return 2;
    case E_V1024SImode: return 4;
    case E_V512DImode: return 8;
    case E_RVVM1HFmode: return 2;
    case E_RVVM2HFmode: return 2;
    case E_RVVMF2HFmode: return 2;
    case E_RVVM1SFmode: return 4;
    case E_RVVM4HFmode: return 2;
    case E_RVVMF4HFmode: return 2;
    case E_RVVM2SFmode: return 4;
    case E_RVVMF2SFmode: return 4;
    case E_RVVM1DFmode: return 8;
    case E_RVVM8HFmode: return 2;
    case E_RVVM4SFmode: return 4;
    case E_RVVM2DFmode: return 8;
    case E_RVVM8SFmode: return 4;
    case E_RVVM4DFmode: return 8;
    case E_RVVM8DFmode: return 8;
    case E_V1HFmode: return 2;
    case E_RVVMF4x2HFmode: return 2;
    case E_RVVMF2x2HFmode: return 2;
    case E_RVVM1x2HFmode: return 2;
    case E_RVVM2x2HFmode: return 2;
    case E_RVVM4x2HFmode: return 2;
    case E_V2HFmode: return 2;
    case E_V1SFmode: return 4;
    case E_RVVMF4x3HFmode: return 2;
    case E_RVVMF2x3HFmode: return 2;
    case E_RVVM1x3HFmode: return 2;
    case E_RVVM2x3HFmode: return 2;
    case E_RVVMF4x4HFmode: return 2;
    case E_RVVMF2x4HFmode: return 2;
    case E_RVVM1x4HFmode: return 2;
    case E_RVVM2x4HFmode: return 2;
    case E_V4HFmode: return 2;
    case E_RVVMF2x2SFmode: return 4;
    case E_RVVM1x2SFmode: return 4;
    case E_RVVM2x2SFmode: return 4;
    case E_RVVM4x2SFmode: return 4;
    case E_V2SFmode: return 4;
    case E_V1DFmode: return 8;
    case E_RVVMF4x5HFmode: return 2;
    case E_RVVMF2x5HFmode: return 2;
    case E_RVVM1x5HFmode: return 2;
    case E_RVVMF4x6HFmode: return 2;
    case E_RVVMF2x6HFmode: return 2;
    case E_RVVM1x6HFmode: return 2;
    case E_RVVMF2x3SFmode: return 4;
    case E_RVVM1x3SFmode: return 4;
    case E_RVVM2x3SFmode: return 4;
    case E_RVVMF4x7HFmode: return 2;
    case E_RVVMF2x7HFmode: return 2;
    case E_RVVM1x7HFmode: return 2;
    case E_RVVMF4x8HFmode: return 2;
    case E_RVVMF2x8HFmode: return 2;
    case E_RVVM1x8HFmode: return 2;
    case E_V8HFmode: return 2;
    case E_RVVMF2x4SFmode: return 4;
    case E_RVVM1x4SFmode: return 4;
    case E_RVVM2x4SFmode: return 4;
    case E_V4SFmode: return 4;
    case E_RVVM1x2DFmode: return 8;
    case E_RVVM2x2DFmode: return 8;
    case E_RVVM4x2DFmode: return 8;
    case E_V2DFmode: return 8;
    case E_RVVMF2x5SFmode: return 4;
    case E_RVVM1x5SFmode: return 4;
    case E_RVVMF2x6SFmode: return 4;
    case E_RVVM1x6SFmode: return 4;
    case E_RVVM1x3DFmode: return 8;
    case E_RVVM2x3DFmode: return 8;
    case E_RVVMF2x7SFmode: return 4;
    case E_RVVM1x7SFmode: return 4;
    case E_V16HFmode: return 2;
    case E_RVVMF2x8SFmode: return 4;
    case E_RVVM1x8SFmode: return 4;
    case E_V8SFmode: return 4;
    case E_RVVM1x4DFmode: return 8;
    case E_RVVM2x4DFmode: return 8;
    case E_V4DFmode: return 8;
    case E_RVVM1x5DFmode: return 8;
    case E_RVVM1x6DFmode: return 8;
    case E_RVVM1x7DFmode: return 8;
    case E_V32HFmode: return 2;
    case E_V16SFmode: return 4;
    case E_RVVM1x8DFmode: return 8;
    case E_V8DFmode: return 8;
    case E_V64HFmode: return 2;
    case E_V32SFmode: return 4;
    case E_V16DFmode: return 8;
    case E_V128HFmode: return 2;
    case E_V64SFmode: return 4;
    case E_V32DFmode: return 8;
    case E_V256HFmode: return 2;
    case E_V128SFmode: return 4;
    case E_V64DFmode: return 8;
    case E_V512HFmode: return 2;
    case E_V256SFmode: return 4;
    case E_V128DFmode: return 8;
    case E_V1024HFmode: return 2;
    case E_V512SFmode: return 4;
    case E_V256DFmode: return 8;
    case E_V2048HFmode: return 2;
    case E_V1024SFmode: return 4;
    case E_V512DFmode: return 8;
    default: return mode_unit_size[mode];
    }
}

#ifdef __cplusplus
inline __attribute__((__always_inline__))
#else
extern __inline__ __attribute__((__always_inline__, __gnu_inline__))
#endif
unsigned short
mode_unit_precision_inline (machine_mode mode)
{
  extern const unsigned short mode_unit_precision[NUM_MACHINE_MODES];
  gcc_assert (mode >= 0 && mode < NUM_MACHINE_MODES);
  switch (mode)
    {
    case E_VOIDmode: return 0;
    case E_BLKmode: return 0;
    case E_CCmode: return 4*BITS_PER_UNIT;
    case E_BImode: return 1;
    case E_QImode: return 1*BITS_PER_UNIT;
    case E_HImode: return 2*BITS_PER_UNIT;
    case E_SImode: return 4*BITS_PER_UNIT;
    case E_DImode: return 8*BITS_PER_UNIT;
    case E_TImode: return 16*BITS_PER_UNIT;
    case E_QQmode: return 1*BITS_PER_UNIT;
    case E_HQmode: return 2*BITS_PER_UNIT;
    case E_SQmode: return 4*BITS_PER_UNIT;
    case E_DQmode: return 8*BITS_PER_UNIT;
    case E_TQmode: return 16*BITS_PER_UNIT;
    case E_UQQmode: return 1*BITS_PER_UNIT;
    case E_UHQmode: return 2*BITS_PER_UNIT;
    case E_USQmode: return 4*BITS_PER_UNIT;
    case E_UDQmode: return 8*BITS_PER_UNIT;
    case E_UTQmode: return 16*BITS_PER_UNIT;
    case E_HAmode: return 2*BITS_PER_UNIT;
    case E_SAmode: return 4*BITS_PER_UNIT;
    case E_DAmode: return 8*BITS_PER_UNIT;
    case E_TAmode: return 16*BITS_PER_UNIT;
    case E_UHAmode: return 2*BITS_PER_UNIT;
    case E_USAmode: return 4*BITS_PER_UNIT;
    case E_UDAmode: return 8*BITS_PER_UNIT;
    case E_UTAmode: return 16*BITS_PER_UNIT;
    case E_HFmode: return 2*BITS_PER_UNIT;
    case E_SFmode: return 4*BITS_PER_UNIT;
    case E_DFmode: return 8*BITS_PER_UNIT;
    case E_TFmode: return 16*BITS_PER_UNIT;
    case E_SDmode: return 4*BITS_PER_UNIT;
    case E_DDmode: return 8*BITS_PER_UNIT;
    case E_TDmode: return 16*BITS_PER_UNIT;
    case E_CQImode: return 1*BITS_PER_UNIT;
    case E_CHImode: return 2*BITS_PER_UNIT;
    case E_CSImode: return 4*BITS_PER_UNIT;
    case E_CDImode: return 8*BITS_PER_UNIT;
    case E_CTImode: return 16*BITS_PER_UNIT;
    case E_HCmode: return 2*BITS_PER_UNIT;
    case E_SCmode: return 4*BITS_PER_UNIT;
    case E_DCmode: return 8*BITS_PER_UNIT;
    case E_TCmode: return 16*BITS_PER_UNIT;
    case E_RVVMF8BImode: return 1;
    case E_RVVMF16BImode: return 1;
    case E_RVVMF32BImode: return 1;
    case E_RVVMF64BImode: return 1;
    case E_V1BImode: return 1;
    case E_V2BImode: return 1;
    case E_V4BImode: return 1;
    case E_V8BImode: return 1;
    case E_RVVMF4BImode: return 1;
    case E_V16BImode: return 1;
    case E_RVVMF2BImode: return 1;
    case E_V32BImode: return 1;
    case E_RVVM1BImode: return 1;
    case E_V64BImode: return 1;
    case E_V128BImode: return 1;
    case E_V256BImode: return 1;
    case E_V512BImode: return 1;
    case E_V1024BImode: return 1;
    case E_V2048BImode: return 1;
    case E_V4096BImode: return 1;
    case E_RVVM1QImode: return 1*BITS_PER_UNIT;
    case E_RVVM2QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF2QImode: return 1*BITS_PER_UNIT;
    case E_RVVM1HImode: return 2*BITS_PER_UNIT;
    case E_RVVM4QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF4QImode: return 1*BITS_PER_UNIT;
    case E_RVVM2HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF2HImode: return 2*BITS_PER_UNIT;
    case E_RVVM1SImode: return 4*BITS_PER_UNIT;
    case E_RVVM8QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF8QImode: return 1*BITS_PER_UNIT;
    case E_RVVM4HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF4HImode: return 2*BITS_PER_UNIT;
    case E_RVVM2SImode: return 4*BITS_PER_UNIT;
    case E_RVVMF2SImode: return 4*BITS_PER_UNIT;
    case E_RVVM1DImode: return 8*BITS_PER_UNIT;
    case E_RVVM8HImode: return 2*BITS_PER_UNIT;
    case E_RVVM4SImode: return 4*BITS_PER_UNIT;
    case E_RVVM2DImode: return 8*BITS_PER_UNIT;
    case E_RVVM8SImode: return 4*BITS_PER_UNIT;
    case E_RVVM4DImode: return 8*BITS_PER_UNIT;
    case E_RVVM8DImode: return 8*BITS_PER_UNIT;
    case E_V1QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF8x2QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF4x2QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF2x2QImode: return 1*BITS_PER_UNIT;
    case E_RVVM1x2QImode: return 1*BITS_PER_UNIT;
    case E_RVVM2x2QImode: return 1*BITS_PER_UNIT;
    case E_RVVM4x2QImode: return 1*BITS_PER_UNIT;
    case E_V2QImode: return 1*BITS_PER_UNIT;
    case E_V1HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF8x3QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF4x3QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF2x3QImode: return 1*BITS_PER_UNIT;
    case E_RVVM1x3QImode: return 1*BITS_PER_UNIT;
    case E_RVVM2x3QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF8x4QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF4x4QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF2x4QImode: return 1*BITS_PER_UNIT;
    case E_RVVM1x4QImode: return 1*BITS_PER_UNIT;
    case E_RVVM2x4QImode: return 1*BITS_PER_UNIT;
    case E_V4QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF4x2HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x2HImode: return 2*BITS_PER_UNIT;
    case E_RVVM1x2HImode: return 2*BITS_PER_UNIT;
    case E_RVVM2x2HImode: return 2*BITS_PER_UNIT;
    case E_RVVM4x2HImode: return 2*BITS_PER_UNIT;
    case E_V2HImode: return 2*BITS_PER_UNIT;
    case E_V1SImode: return 4*BITS_PER_UNIT;
    case E_RVVMF8x5QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF4x5QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF2x5QImode: return 1*BITS_PER_UNIT;
    case E_RVVM1x5QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF8x6QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF4x6QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF2x6QImode: return 1*BITS_PER_UNIT;
    case E_RVVM1x6QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF4x3HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x3HImode: return 2*BITS_PER_UNIT;
    case E_RVVM1x3HImode: return 2*BITS_PER_UNIT;
    case E_RVVM2x3HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF8x7QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF4x7QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF2x7QImode: return 1*BITS_PER_UNIT;
    case E_RVVM1x7QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF8x8QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF4x8QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF2x8QImode: return 1*BITS_PER_UNIT;
    case E_RVVM1x8QImode: return 1*BITS_PER_UNIT;
    case E_V8QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF4x4HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x4HImode: return 2*BITS_PER_UNIT;
    case E_RVVM1x4HImode: return 2*BITS_PER_UNIT;
    case E_RVVM2x4HImode: return 2*BITS_PER_UNIT;
    case E_V4HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x2SImode: return 4*BITS_PER_UNIT;
    case E_RVVM1x2SImode: return 4*BITS_PER_UNIT;
    case E_RVVM2x2SImode: return 4*BITS_PER_UNIT;
    case E_RVVM4x2SImode: return 4*BITS_PER_UNIT;
    case E_V2SImode: return 4*BITS_PER_UNIT;
    case E_V1DImode: return 8*BITS_PER_UNIT;
    case E_RVVMF4x5HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x5HImode: return 2*BITS_PER_UNIT;
    case E_RVVM1x5HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF4x6HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x6HImode: return 2*BITS_PER_UNIT;
    case E_RVVM1x6HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x3SImode: return 4*BITS_PER_UNIT;
    case E_RVVM1x3SImode: return 4*BITS_PER_UNIT;
    case E_RVVM2x3SImode: return 4*BITS_PER_UNIT;
    case E_RVVMF4x7HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x7HImode: return 2*BITS_PER_UNIT;
    case E_RVVM1x7HImode: return 2*BITS_PER_UNIT;
    case E_V16QImode: return 1*BITS_PER_UNIT;
    case E_RVVMF4x8HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x8HImode: return 2*BITS_PER_UNIT;
    case E_RVVM1x8HImode: return 2*BITS_PER_UNIT;
    case E_V8HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x4SImode: return 4*BITS_PER_UNIT;
    case E_RVVM1x4SImode: return 4*BITS_PER_UNIT;
    case E_RVVM2x4SImode: return 4*BITS_PER_UNIT;
    case E_V4SImode: return 4*BITS_PER_UNIT;
    case E_RVVM1x2DImode: return 8*BITS_PER_UNIT;
    case E_RVVM2x2DImode: return 8*BITS_PER_UNIT;
    case E_RVVM4x2DImode: return 8*BITS_PER_UNIT;
    case E_V2DImode: return 8*BITS_PER_UNIT;
    case E_RVVMF2x5SImode: return 4*BITS_PER_UNIT;
    case E_RVVM1x5SImode: return 4*BITS_PER_UNIT;
    case E_RVVMF2x6SImode: return 4*BITS_PER_UNIT;
    case E_RVVM1x6SImode: return 4*BITS_PER_UNIT;
    case E_RVVM1x3DImode: return 8*BITS_PER_UNIT;
    case E_RVVM2x3DImode: return 8*BITS_PER_UNIT;
    case E_RVVMF2x7SImode: return 4*BITS_PER_UNIT;
    case E_RVVM1x7SImode: return 4*BITS_PER_UNIT;
    case E_V32QImode: return 1*BITS_PER_UNIT;
    case E_V16HImode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x8SImode: return 4*BITS_PER_UNIT;
    case E_RVVM1x8SImode: return 4*BITS_PER_UNIT;
    case E_V8SImode: return 4*BITS_PER_UNIT;
    case E_RVVM1x4DImode: return 8*BITS_PER_UNIT;
    case E_RVVM2x4DImode: return 8*BITS_PER_UNIT;
    case E_V4DImode: return 8*BITS_PER_UNIT;
    case E_RVVM1x5DImode: return 8*BITS_PER_UNIT;
    case E_RVVM1x6DImode: return 8*BITS_PER_UNIT;
    case E_RVVM1x7DImode: return 8*BITS_PER_UNIT;
    case E_V64QImode: return 1*BITS_PER_UNIT;
    case E_V32HImode: return 2*BITS_PER_UNIT;
    case E_V16SImode: return 4*BITS_PER_UNIT;
    case E_RVVM1x8DImode: return 8*BITS_PER_UNIT;
    case E_V8DImode: return 8*BITS_PER_UNIT;
    case E_V128QImode: return 1*BITS_PER_UNIT;
    case E_V64HImode: return 2*BITS_PER_UNIT;
    case E_V32SImode: return 4*BITS_PER_UNIT;
    case E_V16DImode: return 8*BITS_PER_UNIT;
    case E_V256QImode: return 1*BITS_PER_UNIT;
    case E_V128HImode: return 2*BITS_PER_UNIT;
    case E_V64SImode: return 4*BITS_PER_UNIT;
    case E_V32DImode: return 8*BITS_PER_UNIT;
    case E_V512QImode: return 1*BITS_PER_UNIT;
    case E_V256HImode: return 2*BITS_PER_UNIT;
    case E_V128SImode: return 4*BITS_PER_UNIT;
    case E_V64DImode: return 8*BITS_PER_UNIT;
    case E_V1024QImode: return 1*BITS_PER_UNIT;
    case E_V512HImode: return 2*BITS_PER_UNIT;
    case E_V256SImode: return 4*BITS_PER_UNIT;
    case E_V128DImode: return 8*BITS_PER_UNIT;
    case E_V2048QImode: return 1*BITS_PER_UNIT;
    case E_V1024HImode: return 2*BITS_PER_UNIT;
    case E_V512SImode: return 4*BITS_PER_UNIT;
    case E_V256DImode: return 8*BITS_PER_UNIT;
    case E_V4096QImode: return 1*BITS_PER_UNIT;
    case E_V2048HImode: return 2*BITS_PER_UNIT;
    case E_V1024SImode: return 4*BITS_PER_UNIT;
    case E_V512DImode: return 8*BITS_PER_UNIT;
    case E_RVVM1HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM2HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF2HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM1SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM4HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF4HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM2SFmode: return 4*BITS_PER_UNIT;
    case E_RVVMF2SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM1DFmode: return 8*BITS_PER_UNIT;
    case E_RVVM8HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM4SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM2DFmode: return 8*BITS_PER_UNIT;
    case E_RVVM8SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM4DFmode: return 8*BITS_PER_UNIT;
    case E_RVVM8DFmode: return 8*BITS_PER_UNIT;
    case E_V1HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF4x2HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x2HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM1x2HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM2x2HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM4x2HFmode: return 2*BITS_PER_UNIT;
    case E_V2HFmode: return 2*BITS_PER_UNIT;
    case E_V1SFmode: return 4*BITS_PER_UNIT;
    case E_RVVMF4x3HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x3HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM1x3HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM2x3HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF4x4HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x4HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM1x4HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM2x4HFmode: return 2*BITS_PER_UNIT;
    case E_V4HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x2SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM1x2SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM2x2SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM4x2SFmode: return 4*BITS_PER_UNIT;
    case E_V2SFmode: return 4*BITS_PER_UNIT;
    case E_V1DFmode: return 8*BITS_PER_UNIT;
    case E_RVVMF4x5HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x5HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM1x5HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF4x6HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x6HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM1x6HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x3SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM1x3SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM2x3SFmode: return 4*BITS_PER_UNIT;
    case E_RVVMF4x7HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x7HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM1x7HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF4x8HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x8HFmode: return 2*BITS_PER_UNIT;
    case E_RVVM1x8HFmode: return 2*BITS_PER_UNIT;
    case E_V8HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x4SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM1x4SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM2x4SFmode: return 4*BITS_PER_UNIT;
    case E_V4SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM1x2DFmode: return 8*BITS_PER_UNIT;
    case E_RVVM2x2DFmode: return 8*BITS_PER_UNIT;
    case E_RVVM4x2DFmode: return 8*BITS_PER_UNIT;
    case E_V2DFmode: return 8*BITS_PER_UNIT;
    case E_RVVMF2x5SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM1x5SFmode: return 4*BITS_PER_UNIT;
    case E_RVVMF2x6SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM1x6SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM1x3DFmode: return 8*BITS_PER_UNIT;
    case E_RVVM2x3DFmode: return 8*BITS_PER_UNIT;
    case E_RVVMF2x7SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM1x7SFmode: return 4*BITS_PER_UNIT;
    case E_V16HFmode: return 2*BITS_PER_UNIT;
    case E_RVVMF2x8SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM1x8SFmode: return 4*BITS_PER_UNIT;
    case E_V8SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM1x4DFmode: return 8*BITS_PER_UNIT;
    case E_RVVM2x4DFmode: return 8*BITS_PER_UNIT;
    case E_V4DFmode: return 8*BITS_PER_UNIT;
    case E_RVVM1x5DFmode: return 8*BITS_PER_UNIT;
    case E_RVVM1x6DFmode: return 8*BITS_PER_UNIT;
    case E_RVVM1x7DFmode: return 8*BITS_PER_UNIT;
    case E_V32HFmode: return 2*BITS_PER_UNIT;
    case E_V16SFmode: return 4*BITS_PER_UNIT;
    case E_RVVM1x8DFmode: return 8*BITS_PER_UNIT;
    case E_V8DFmode: return 8*BITS_PER_UNIT;
    case E_V64HFmode: return 2*BITS_PER_UNIT;
    case E_V32SFmode: return 4*BITS_PER_UNIT;
    case E_V16DFmode: return 8*BITS_PER_UNIT;
    case E_V128HFmode: return 2*BITS_PER_UNIT;
    case E_V64SFmode: return 4*BITS_PER_UNIT;
    case E_V32DFmode: return 8*BITS_PER_UNIT;
    case E_V256HFmode: return 2*BITS_PER_UNIT;
    case E_V128SFmode: return 4*BITS_PER_UNIT;
    case E_V64DFmode: return 8*BITS_PER_UNIT;
    case E_V512HFmode: return 2*BITS_PER_UNIT;
    case E_V256SFmode: return 4*BITS_PER_UNIT;
    case E_V128DFmode: return 8*BITS_PER_UNIT;
    case E_V1024HFmode: return 2*BITS_PER_UNIT;
    case E_V512SFmode: return 4*BITS_PER_UNIT;
    case E_V256DFmode: return 8*BITS_PER_UNIT;
    case E_V2048HFmode: return 2*BITS_PER_UNIT;
    case E_V1024SFmode: return 4*BITS_PER_UNIT;
    case E_V512DFmode: return 8*BITS_PER_UNIT;
    default: return mode_unit_precision[mode];
    }
}

#endif /* GCC_VERSION >= 4001 */

#endif /* insn-modes-inline.h */
